package net.minecraft.src.railcraft.common.api;

import java.util.ArrayList;
import java.util.List;
import net.minecraft.src.AxisAlignedBB;
import net.minecraft.src.BlockRail;
import net.minecraft.src.EntityMinecart;
import net.minecraft.src.MathHelper;
import net.minecraft.src.ModLoader;
import net.minecraft.src.TileEntity;
import net.minecraft.src.World;
import net.minecraft.src.forge.MinecraftForge;
import net.minecraft.src.forge.NetworkMod;

public class CartTools
{

    public static void registerMinecart(NetworkMod mod, Class<? extends EntityMinecart> type, String tag, int entityId, int internalId)
    {
        ModLoader.registerEntityID(type, tag, entityId);
        MinecraftForge.registerEntity(type, mod, internalId, 80, 2, true);
    }

    public static boolean isMinecartOnRailAt(World world, int i, int j, int k, float bounds)
    {
        return isMinecartOnRailAt(world, i, j, k, bounds, null, true);
    }

    public static boolean isMinecartOnRailAt(World world, int i, int j, int k, float bounds, Class<? extends EntityMinecart> type, boolean subclass)
    {
        if(BlockRail.isRailBlockAt(world, i, j, k)) {
            return isMinecartAt(world, i, j, k, bounds, type, subclass);
        }
        return false;
    }

    public static boolean isMinecartOnAnySide(World world, int i, int j, int k, float bounds)
    {
        return isMinecartOnAnySide(world, i, j, k, bounds, null, true);
    }

    public static boolean isMinecartOnAnySide(World world, int i, int j, int k, float bounds, Class<? extends EntityMinecart> type, boolean subclass)
    {
        List<EntityMinecart> list = new ArrayList<EntityMinecart>();
        for(int side = 0; side < 6; side++) {
            list.addAll(getMinecartsOnSide(world, i, j, k, bounds, side));
        }

        if(type == null) {
            return !list.isEmpty();
        } else {
            for(EntityMinecart cart : list) {
                if((subclass && type.isInstance(cart)) || cart.getClass() == type) {
                    return true;
                }
            }
        }
        return false;
    }

    public static boolean isMinecartAt(World world, int i, int j, int k, float bounds)
    {
        return isMinecartAt(world, i, j, k, bounds, null, true);
    }

    public static boolean isMinecartAt(World world, int i, int j, int k, float bounds, Class<? extends EntityMinecart> type, boolean subclass)
    {
        List<EntityMinecart> list = getMinecartsAt(world, i, j, k, bounds);

        if(type == null) {
            return !list.isEmpty();
        } else {
            for(EntityMinecart cart : list) {
                if((subclass && type.isInstance(cart)) || cart.getClass() == type) {
                    return true;
                }
            }
        }
        return false;
    }

    public static List<EntityMinecart> getMinecartsOnAllSides(World world, int i, int j, int k, float bounds)
    {
        List<EntityMinecart> carts = new ArrayList<EntityMinecart>();
        for(int side = 0; side < 6; side++) {
            carts.addAll(getMinecartsOnSide(world, i, j, k, bounds, side));
        }

        return carts;
    }

    public static List<EntityMinecart> getMinecartsOnAllSides(World world, int i, int j, int k, float bounds, Class<? extends EntityMinecart> type, boolean subclass)
    {
        List<EntityMinecart> list = new ArrayList<EntityMinecart>();
        List<EntityMinecart> carts = new ArrayList<EntityMinecart>();
        for(int side = 0; side < 6; side++) {
            list.addAll(getMinecartsOnSide(world, i, j, k, bounds, side));
        }

        for(EntityMinecart cart : list) {
            if((subclass && type.isInstance(cart)) || cart.getClass() == type) {
                carts.add(cart);
            }
        }
        return carts;
    }

    public static List<EntityMinecart> getMinecartsOnSide(World world, int i, int j, int k, float bounds, int side)
    {
        return getMinecartsAt(world, GeneralTools.getXOnSide(i, side), GeneralTools.getYOnSide(j, side), GeneralTools.getZOnSide(k, side), bounds);
    }

    public static EntityMinecart getMinecartOnSide(World world, int i, int j, int k, float bounds, int side)
    {
        for(EntityMinecart cart : getMinecartsOnSide(world, i, j, k, bounds, side)) {
            return cart;
        }
        return null;
    }

    public static EntityMinecart getMinecartOnSide(World world, int i, int j, int k, float bounds, int side, Class<? extends EntityMinecart> type, boolean subclass)
    {
        for(EntityMinecart cart : getMinecartsOnSide(world, i, j, k, bounds, side)) {
            if(type == null || (subclass && type.isInstance(cart)) || cart.getClass() == type) {
                return cart;
            }
        }
        return null;
    }

    public static List<EntityMinecart> getMinecartsAt(World world, int i, int j, int k, float bounds)
    {
        List entities = world.getEntitiesWithinAABB(net.minecraft.src.EntityMinecart.class, AxisAlignedBB.getBoundingBoxFromPool(i + bounds, j + bounds, k + bounds, i + 1 - bounds, j + 1 - bounds, k + 1 - bounds));
        List<EntityMinecart> carts = new ArrayList<EntityMinecart>();
        for(Object o : entities) {
            carts.add((EntityMinecart)o);
        }
        return carts;
    }

    public static List<EntityMinecart> getMinecartsIn(World world, int i1, int j1, int k1, int i2, int j2, int k2)
    {
        List entities = world.getEntitiesWithinAABB(net.minecraft.src.EntityMinecart.class, AxisAlignedBB.getBoundingBoxFromPool(i1, j1, k1, i2, j2, k2));
        List<EntityMinecart> carts = new ArrayList<EntityMinecart>();
        for(Object o : entities) {
            carts.add((EntityMinecart)o);
        }
        return carts;
    }

    /**
     * Returns the cart's "speed". It is not capped by the carts max speed,
     * it instead returns the cart "potential" speed.
     * Used by collision and linkage logic.
     * Do not use this to determine how fast a cart is currently moving.
     * @param cart
     * @return speed
     */
    public static double getCartSpeedUncapped(EntityMinecart cart)
    {
        return Math.sqrt(cart.motionX * cart.motionX + cart.motionZ * cart.motionZ);
    }

    public static boolean cartVelocityIsLessThan(EntityMinecart cart, float vel)
    {
        return Math.abs(cart.motionX) < vel && Math.abs(cart.motionZ) < vel;
    }

    /**
     * Checks to see if a cart is being held by a ILockdownRail.
     * @param cart The cart to check
     * @return True if being held
     */
    public static boolean isCartLockedDown(EntityMinecart cart)
    {
        int x = MathHelper.floor_double(cart.posX);
        int y = MathHelper.floor_double(cart.posY);
        int z = MathHelper.floor_double(cart.posZ);

        if(BlockRail.isRailBlockAt(cart.worldObj, x, y - 1, z)) {
            --y;
        }

        TileEntity rail = cart.worldObj.getBlockTileEntity(x, y, z);
        return rail instanceof ILockdownRail && ((ILockdownRail)rail).isCartLockedDown(cart);
    }
}
