package net.minecraft.src.railcraft.common.api;

import net.minecraft.src.Block;
import net.minecraft.src.BlockRail;
import net.minecraft.src.TileEntity;
import net.minecraft.src.World;

public class RailTools
{

    public static boolean isSpeedRailAt(World world, int i, int j, int k)
    {
        int id = world.getBlockId(i, j, k);
        Block block = Block.blocksList[id];
        if(block instanceof IRailSpeed) {
            return true;
        }
        TileEntity tile = world.getBlockTileEntity(i, j, k);
        return tile instanceof IRailSpeed;
    }

    public static boolean isNeighborRailUpSlope(World world, int i, int j, int k)
    {
        for(int side = 2; side < 6; side++) {
            int jj = j;
            int id = GeneralTools.getBlockIdOnSide(world, i, jj, k, side);
            if(!BlockRail.isRailBlock(id)) {
                j++;
                id = GeneralTools.getBlockIdOnSide(world, i, jj, k, side);
                if(!BlockRail.isRailBlock(id)) {
                    continue;
                }
            }
            int meta = GeneralTools.getBlockMetadataFromSide(world, i, jj, k, side);
            if(meta == 2 || meta == 3 || meta == 4 || meta == 5) {
                return true;
            }
        }
        return false;
    }

    public static boolean areDistantRailsConnectedAlongAxis(World world, int i1, int j1, int k1, int i2, int j2, int k2)
    {
        if(j1 < 0 || j2 < 0) {
            return false;
        }
        if(i1 != i2 && k1 != k2) {
            return false;
        }
        if(i1 != i2) {
            int min = 0;
            int max = 0;
            int jj = 0;
            if(i1 < i2) {
                min = i1;
                max = i2;
                jj = j1;
            } else {
                min = i2;
                max = i1;
                jj = j2;
            }
            for(int ii = min; ii <= max; ii++) {
                if(world.blockExists(ii, jj, k1)) {
                    if(BlockRail.isRailBlockAt(world, ii, jj, k1)) {
                        continue;
                    } else if(BlockRail.isRailBlockAt(world, ii, jj - 1, k1)) {
                        jj--;
                        continue;
                    } else if(BlockRail.isRailBlockAt(world, ii, jj + 1, k1)) {
                        jj++;
                        continue;
                    } else {
                        return false;
                    }
                }
            }
        } else if(k1 != k2) {
            int min = 0;
            int max = 0;
            int jj = 0;
            if(k1 < k2) {
                min = k1;
                max = k2;
                jj = j1;
            } else {
                min = k2;
                max = k1;
                jj = j2;
            }
            for(int kk = min; kk <= max; kk++) {
                if(world.blockExists(i1, jj, kk)) {
                    if(BlockRail.isRailBlockAt(world, i1, jj, kk)) {
                        continue;
                    } else if(BlockRail.isRailBlockAt(world, i1, jj - 1, kk)) {
                        jj--;
                        continue;
                    } else if(BlockRail.isRailBlockAt(world, i1, jj + 1, kk)) {
                        jj++;
                        continue;
                    } else {
                        return false;
                    }
                }
            }
        }
        return true;
    }
}
